aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorJung-uk Kim <jkim@FreeBSD.org>2019-07-09 17:17:45 +0000
committerJung-uk Kim <jkim@FreeBSD.org>2019-07-09 17:17:45 +0000
commit3d51590acd9ce6cfcd823e3d4d340f04ff800228 (patch)
treed8c7a85f644b22ff2a5e5c9a56190e6798fb381e
parent08f4234e06d8d6855c0b79c95da98f267ca3554e (diff)
downloadsrc-3d51590acd9ce6cfcd823e3d4d340f04ff800228.tar.gz
src-3d51590acd9ce6cfcd823e3d4d340f04ff800228.zip
Import ACPICA 20190703.vendor/acpica/20190703
Notes
Notes: svn path=/vendor-sys/acpica/dist/; revision=349861 svn path=/vendor-sys/acpica/20190703/; revision=349862; tag=vendor/acpica/20190703
-rw-r--r--changes.txt49
-rw-r--r--source/compiler/asldefine.h16
-rw-r--r--source/compiler/aslglobal.h15
-rw-r--r--source/compiler/aslload.c59
-rw-r--r--source/compiler/aslmessages.c3
-rw-r--r--source/compiler/aslmessages.h1
-rw-r--r--source/compiler/aslstubs.c6
-rw-r--r--source/compiler/aslsupport.l18
-rw-r--r--source/components/dispatcher/dsinit.c2
-rw-r--r--source/components/events/evgpe.c9
-rw-r--r--source/components/events/evgpeblk.c2
-rw-r--r--source/components/events/evxface.c2
-rw-r--r--source/components/events/evxfgpe.c2
-rw-r--r--source/components/executer/exconfig.c12
-rw-r--r--source/components/namespace/nsaccess.c57
-rw-r--r--source/components/namespace/nseval.c210
-rw-r--r--source/components/namespace/nsinit.c51
-rw-r--r--source/components/namespace/nsload.c12
-rw-r--r--source/components/namespace/nsutils.c13
-rw-r--r--source/components/tables/tbdata.c13
-rw-r--r--source/components/tables/tbxfload.c7
-rw-r--r--source/components/utilities/utinit.c1
-rw-r--r--source/components/utilities/utxfinit.c18
-rw-r--r--source/include/acevents.h3
-rw-r--r--source/include/acglobal.h1
-rw-r--r--source/include/acnamesp.h4
-rw-r--r--source/include/acpixf.h2
-rw-r--r--source/include/platform/acmsvc.h15
-rw-r--r--source/include/platform/acwin.h31
-rw-r--r--source/include/platform/acwin64.h22
-rw-r--r--source/tools/acpiexec/aetests.c8
-rw-r--r--source/tools/acpinames/anstubs.c7
-rw-r--r--source/tools/acpixtract/axmain.c2
33 files changed, 319 insertions, 354 deletions
diff --git a/changes.txt b/changes.txt
index 6d7724949565..0020457b5d2a 100644
--- a/changes.txt
+++ b/changes.txt
@@ -1,4 +1,53 @@
----------------------------------------
+03 July 2019. Summary of changes for version 20190703:
+
+
+1) ACPICA kernel-resident subsystem:
+
+Remove legacy module-level support code. There were still some remnants
+of the legacy module-level code executions. Since we no longer support
+this option, this is essentially dead code and has been removed from the
+ACPICA source.
+
+iASL: ensure that _WAK, _PTS, _TTS, and _Sx are declared only at the root
+scope. If these named objects are declared outside the root scope, they
+will not be invoked by any host Operating System.
+
+Clear status of GPEs on first direct enable. ACPI GPEs (other than the EC
+one) can be enabled in two situations. First, the GPEs with existing _Lxx
+and _Exx methods are enabled implicitly by ACPICA during system
+initialization. Second, the GPEs without these methods (like GPEs listed
+by _PRW objects for wakeup devices) need to be enabled directly by the
+code that is going to use them (e.g. ACPI power management or device
+drivers).
+
+In the former case, if the status of a given GPE is set to start with,
+its handler method (either _Lxx or _Exx) needs to be invoked to take care
+of the events (possibly) signaled before the GPE was enabled. In the
+latter case, however, the first caller of AcpiEnableGpe() for a given GPE
+should not be expected to care about any events that might be signaled
+through it earlier. In that case, it is better to clear the status of
+the GPE before enabling it, to prevent stale events from triggering
+unwanted actions (like spurious system resume, for example).
+
+For this reason, modify AcpiEvAddGpeReference() to take an additional
+boolean argument indicating whether or not the GPE status needs to be
+cleared when its reference counter changes from zero to one and make
+AcpiEnableGpe() pass TRUE to it through that new argument.
+
+
+2) iASL Compiler/Disassembler and ACPICA tools:
+
+The tool generation process has been migrated to MSVC 2017, and all
+project files have been upgraded. The new project files appear in the
+directory \acpica\generate\msvc2017. This change effectively deprecates
+the older project files in \acpica\generate\msvc9.
+
+iASL: ensure that _WAK, _PTS, _TTS, and _Sx are declared only at the root
+scope. If these named objects are declared outside the root scope, they
+will not be invoked by any host Operating System
+
+----------------------------------------
09 May 2019. Summary of changes for version 20190509:
diff --git a/source/compiler/asldefine.h b/source/compiler/asldefine.h
index cffb0af1fe52..c511187529ed 100644
--- a/source/compiler/asldefine.h
+++ b/source/compiler/asldefine.h
@@ -298,4 +298,20 @@
#define COMMENT_CAPTURE_ON AslGbl_CommentState.CaptureComments = TRUE;
#define COMMENT_CAPTURE_OFF AslGbl_CommentState.CaptureComments = FALSE;
+/*
+ * Special name segments - these must only be declared at the root scope
+ */
+#define NAMESEG__PTS "_PTS"
+#define NAMESEG__WAK "_WAK"
+#define NAMESEG__S0 "_S0_"
+#define NAMESEG__S1 "_S1_"
+#define NAMESEG__S2 "_S2_"
+#define NAMESEG__S3 "_S3_"
+#define NAMESEG__S4 "_S4_"
+#define NAMESEG__S5 "_S5_"
+#define NAMESEG__TTS "_TTS"
+
+#define MAX_SPECIAL_NAMES 9
+
+
#endif /* ASLDEFINE.H */
diff --git a/source/compiler/aslglobal.h b/source/compiler/aslglobal.h
index 213c540875f5..740f7f34ae51 100644
--- a/source/compiler/aslglobal.h
+++ b/source/compiler/aslglobal.h
@@ -223,13 +223,28 @@ const char *AslGbl_OpFlagNames[ACPI_NUM_OP_FLAGS] =
"OP_NOT_FOUND_DURING_LOAD"
};
+const char *AslGbl_SpecialNamedObjects [MAX_SPECIAL_NAMES] =
+{
+ NAMESEG__PTS,
+ NAMESEG__WAK,
+ NAMESEG__S0,
+ NAMESEG__S1,
+ NAMESEG__S2,
+ NAMESEG__S3,
+ NAMESEG__S4,
+ NAMESEG__S5,
+ NAMESEG__TTS
+};
+
#else
extern ASL_FILE_DESC AslGbl_FileDescs [ASL_NUM_FILES];
extern UINT32 AslGbl_ExceptionCount[ASL_NUM_REPORT_LEVELS];
extern const char *AslGbl_OpFlagNames[ACPI_NUM_OP_FLAGS];
+extern const char *AslGbl_SpecialNamedObjects[MAX_SPECIAL_NAMES];
#endif
+
/*
* Parser and other externals
*/
diff --git a/source/compiler/aslmessages.c b/source/compiler/aslmessages.c
index cf7a7a4f47cb..48c4dbabacef 100644
--- a/source/compiler/aslmessages.c
+++ b/source/compiler/aslmessages.c
@@ -365,7 +365,8 @@ const char *AslCompilerMsgs [] =
/* ASL_MSG_REGION_LENGTH */ "Operation Region declared with zero length",
/* ASL_MSG_TEMPORARY_OBJECT */ "Object is created temporarily in another method and cannot be accessed",
/* ASL_MSG_UNDEFINED_EXTERNAL */ "Named object was declared external but the actual definition does not exist",
-/* ASL_MSG_BUFFER_FIELD_OVERFLOW */ "Buffer field extends beyond end of target buffer"
+/* ASL_MSG_BUFFER_FIELD_OVERFLOW */ "Buffer field extends beyond end of target buffer",
+/* ASL_MSG_INVALID_SPECIAL_NAME */ "declaration of this named object outside root scope is illegal"
};
/* Table compiler */
diff --git a/source/compiler/aslmessages.h b/source/compiler/aslmessages.h
index 15613685d192..d998f0fe8eda 100644
--- a/source/compiler/aslmessages.h
+++ b/source/compiler/aslmessages.h
@@ -368,6 +368,7 @@ typedef enum
ASL_MSG_TEMPORARY_OBJECT,
ASL_MSG_UNDEFINED_EXTERNAL,
ASL_MSG_BUFFER_FIELD_OVERFLOW,
+ ASL_MSG_INVALID_SPECIAL_NAME,
/* These messages are used by the Data Table compiler only */
diff --git a/source/compiler/aslstubs.c b/source/compiler/aslstubs.c
index a54c498c479c..a8f22bd8c803 100644
--- a/source/compiler/aslstubs.c
+++ b/source/compiler/aslstubs.c
@@ -166,12 +166,6 @@
* Things like Events, Global Lock, etc. are not used
* by the compiler, so they are stubbed out here.
*/
-void
-AcpiNsExecModuleCodeList (
- void)
-{
-}
-
ACPI_STATUS
AcpiNsInitializeObjects (
void)
diff --git a/source/compiler/aslsupport.l b/source/compiler/aslsupport.l
index 69c1399a9ec4..297bf2ee6bcc 100644
--- a/source/compiler/aslsupport.l
+++ b/source/compiler/aslsupport.l
@@ -220,7 +220,7 @@ AslDoLineDirective (
while ((c = input()) != '\n' && c != EOF)
{
- *AslGbl_LineBufPtr = c;
+ *AslGbl_LineBufPtr = (char) c;
AslGbl_LineBufPtr++;
}
*AslGbl_LineBufPtr = 0;
@@ -498,7 +498,7 @@ AslInsertLineBuffer (
if (AcpiGbl_CaptureComments)
{
- CvProcessCommentState (SourceChar);
+ CvProcessCommentState ((char) SourceChar);
}
}
}
@@ -601,7 +601,7 @@ loop:
AslInsertLineBuffer (c);
if (AcpiGbl_CaptureComments && CurrentState.CaptureComments)
{
- *StringBuffer = c;
+ *StringBuffer = (char) c;
++StringBuffer;
}
c1 = c;
@@ -629,7 +629,7 @@ loop:
AslInsertLineBuffer (c);
if (AcpiGbl_CaptureComments && CurrentState.CaptureComments)
{
- *StringBuffer = c;
+ *StringBuffer = (char) c;
++StringBuffer;
}
@@ -720,7 +720,7 @@ AslDoCommentType2 (
AslInsertLineBuffer (c);
if (AcpiGbl_CaptureComments && CurrentState.CaptureComments)
{
- *StringBuffer = c;
+ *StringBuffer = (char) c;
++StringBuffer;
}
}
@@ -878,7 +878,7 @@ DoCharacter:
if (ACPI_IS_OCTAL_DIGIT (StringChar))
{
State = ASL_OCTAL_CONSTANT;
- ConvertBuffer[0] = StringChar;
+ ConvertBuffer[0] = (char) StringChar;
i = 1;
continue;
}
@@ -934,7 +934,7 @@ DoCharacter:
/* Append another digit of the constant */
- ConvertBuffer[i] = StringChar;
+ ConvertBuffer[i] = (char) StringChar;
i++;
continue;
@@ -978,7 +978,7 @@ DoCharacter:
/* Append another digit of the constant */
- ConvertBuffer[i] = StringChar;
+ ConvertBuffer[i] = (char) StringChar;
i++;
continue;
@@ -989,7 +989,7 @@ DoCharacter:
/* Save the finished character */
- *StringBuffer = StringChar;
+ *StringBuffer = (char) StringChar;
StringBuffer++;
if (StringBuffer >= EndBuffer)
{
diff --git a/source/components/dispatcher/dsinit.c b/source/components/dispatcher/dsinit.c
index e7045f461a8a..9810f9ff3f71 100644
--- a/source/components/dispatcher/dsinit.c
+++ b/source/components/dispatcher/dsinit.c
@@ -362,7 +362,7 @@ AcpiDsInitializeObjects (
if (ACPI_COMPARE_NAMESEG (Table->Signature, ACPI_SIG_DSDT))
{
ACPI_DEBUG_PRINT_RAW ((ACPI_DB_INIT,
- "\nInitializing Namespace objects:\n"));
+ "\nACPI table initialization:\n"));
}
/* Summary of objects initialized */
diff --git a/source/components/events/evgpe.c b/source/components/events/evgpe.c
index f5ea992a270b..a7ee84c92f11 100644
--- a/source/components/events/evgpe.c
+++ b/source/components/events/evgpe.c
@@ -316,6 +316,7 @@ AcpiEvMaskGpe (
* FUNCTION: AcpiEvAddGpeReference
*
* PARAMETERS: GpeEventInfo - Add a reference to this GPE
+ * ClearOnEnable - Clear GPE status before enabling it
*
* RETURN: Status
*
@@ -326,7 +327,8 @@ AcpiEvMaskGpe (
ACPI_STATUS
AcpiEvAddGpeReference (
- ACPI_GPE_EVENT_INFO *GpeEventInfo)
+ ACPI_GPE_EVENT_INFO *GpeEventInfo,
+ BOOLEAN ClearOnEnable)
{
ACPI_STATUS Status = AE_OK;
@@ -344,6 +346,11 @@ AcpiEvAddGpeReference (
{
/* Enable on first reference */
+ if (ClearOnEnable)
+ {
+ (void) AcpiHwClearGpe (GpeEventInfo);
+ }
+
Status = AcpiEvUpdateGpeEnableMask (GpeEventInfo);
if (ACPI_SUCCESS (Status))
{
diff --git a/source/components/events/evgpeblk.c b/source/components/events/evgpeblk.c
index 7e1dd1960cc8..80157dcd362f 100644
--- a/source/components/events/evgpeblk.c
+++ b/source/components/events/evgpeblk.c
@@ -637,7 +637,7 @@ AcpiEvInitializeGpeBlock (
continue;
}
- Status = AcpiEvAddGpeReference (GpeEventInfo);
+ Status = AcpiEvAddGpeReference (GpeEventInfo, FALSE);
if (ACPI_FAILURE (Status))
{
ACPI_EXCEPTION ((AE_INFO, Status,
diff --git a/source/components/events/evxface.c b/source/components/events/evxface.c
index 8577a6d02d82..8e8eacf70224 100644
--- a/source/components/events/evxface.c
+++ b/source/components/events/evxface.c
@@ -1256,7 +1256,7 @@ AcpiRemoveGpeHandler (
ACPI_GPE_DISPATCH_NOTIFY)) &&
Handler->OriginallyEnabled)
{
- (void) AcpiEvAddGpeReference (GpeEventInfo);
+ (void) AcpiEvAddGpeReference (GpeEventInfo, FALSE);
if (ACPI_GPE_IS_POLLING_NEEDED (GpeEventInfo))
{
/* Poll edge triggered GPEs to handle existing events */
diff --git a/source/components/events/evxfgpe.c b/source/components/events/evxfgpe.c
index 52061db190cc..1dc0fe1d2963 100644
--- a/source/components/events/evxfgpe.c
+++ b/source/components/events/evxfgpe.c
@@ -267,7 +267,7 @@ AcpiEnableGpe (
if (ACPI_GPE_DISPATCH_TYPE (GpeEventInfo->Flags) !=
ACPI_GPE_DISPATCH_NONE)
{
- Status = AcpiEvAddGpeReference (GpeEventInfo);
+ Status = AcpiEvAddGpeReference (GpeEventInfo, TRUE);
if (ACPI_SUCCESS (Status) &&
ACPI_GPE_IS_POLLING_NEEDED (GpeEventInfo))
{
diff --git a/source/components/executer/exconfig.c b/source/components/executer/exconfig.c
index 84a058adaf7b..eba1a6d28ba3 100644
--- a/source/components/executer/exconfig.c
+++ b/source/components/executer/exconfig.c
@@ -342,10 +342,9 @@ AcpiExLoadTableOp (
return_ACPI_STATUS (Status);
}
- /* Complete the initialization/resolution of package objects */
+ /* Complete the initialization/resolution of new objects */
- Status = AcpiNsWalkNamespace (ACPI_TYPE_PACKAGE, ACPI_ROOT_OBJECT,
- ACPI_UINT32_MAX, 0, AcpiNsInitOnePackage, NULL, NULL, NULL);
+ AcpiNsInitializeObjects ();
/* Parameter Data (optional) */
@@ -620,10 +619,11 @@ AcpiExLoadOp (
return_ACPI_STATUS (Status);
}
- /* Complete the initialization/resolution of package objects */
+ /* Complete the initialization/resolution of new objects */
- Status = AcpiNsWalkNamespace (ACPI_TYPE_PACKAGE, ACPI_ROOT_OBJECT,
- ACPI_UINT32_MAX, 0, AcpiNsInitOnePackage, NULL, NULL, NULL);
+ AcpiExExitInterpreter ();
+ AcpiNsInitializeObjects ();
+ AcpiExEnterInterpreter ();
/* Store the DdbHandle into the Target operand */
diff --git a/source/components/namespace/nsaccess.c b/source/components/namespace/nsaccess.c
index 94dba1307a59..4be555c080e8 100644
--- a/source/components/namespace/nsaccess.c
+++ b/source/components/namespace/nsaccess.c
@@ -184,6 +184,7 @@ AcpiNsRootInitialize (
ACPI_STATUS Status;
const ACPI_PREDEFINED_NAMES *InitVal = NULL;
ACPI_NAMESPACE_NODE *NewNode;
+ ACPI_NAMESPACE_NODE *PrevNode = NULL;
ACPI_OPERAND_OBJECT *ObjDesc;
ACPI_STRING Val = NULL;
@@ -213,13 +214,30 @@ AcpiNsRootInitialize (
*/
AcpiGbl_RootNode = &AcpiGbl_RootNodeStruct;
- /* Enter the pre-defined names in the name table */
+ /* Enter the predefined names in the name table */
ACPI_DEBUG_PRINT ((ACPI_DB_INFO,
"Entering predefined entries into namespace\n"));
+ /*
+ * Create the initial (default) namespace.
+ * This namespace looks like something similar to this:
+ *
+ * ACPI Namespace (from Namespace Root):
+ * 0 _GPE Scope 00203160 00
+ * 0 _PR_ Scope 002031D0 00
+ * 0 _SB_ Device 00203240 00 Notify Object: 0020ADD8
+ * 0 _SI_ Scope 002032B0 00
+ * 0 _TZ_ Device 00203320 00
+ * 0 _REV Integer 00203390 00 = 0000000000000002
+ * 0 _OS_ String 00203488 00 Len 14 "Microsoft Windows NT"
+ * 0 _GL_ Mutex 00203580 00 Object 002035F0
+ * 0 _OSI Method 00203678 00 Args 1 Len 0000 Aml 00000000
+ */
for (InitVal = AcpiGbl_PreDefinedNames; InitVal->Name; InitVal++)
{
+ Status = AE_OK;
+
/* _OSI is optional for now, will be permanent later */
if (!strcmp (InitVal->Name, "_OSI") && !AcpiGbl_CreateOsiMethod)
@@ -227,17 +245,35 @@ AcpiNsRootInitialize (
continue;
}
- Status = AcpiNsLookup (NULL, ACPI_CAST_PTR (char, InitVal->Name),
- InitVal->Type, ACPI_IMODE_LOAD_PASS2, ACPI_NS_NO_UPSEARCH,
- NULL, &NewNode);
- if (ACPI_FAILURE (Status))
+ /*
+ * Create, init, and link the new predefined name
+ * Note: No need to use AcpiNsLookup here because all the
+ * predefined names are at the root level. It is much easier to
+ * just create and link the new node(s) here.
+ */
+ NewNode = ACPI_ALLOCATE_ZEROED (sizeof (ACPI_NAMESPACE_NODE));
+ if (!NewNode)
{
- ACPI_EXCEPTION ((AE_INFO, Status,
- "Could not create predefined name %s",
- InitVal->Name));
- continue;
+ Status = AE_NO_MEMORY;
+ goto UnlockAndExit;
}
+ ACPI_COPY_NAMESEG (NewNode->Name.Ascii, InitVal->Name);
+ NewNode->DescriptorType = ACPI_DESC_TYPE_NAMED;
+ NewNode->Type = InitVal->Type;
+
+ if (!PrevNode)
+ {
+ AcpiGbl_RootNodeStruct.Child = NewNode;
+ }
+ else
+ {
+ PrevNode->Peer = NewNode;
+ }
+
+ NewNode->Parent = &AcpiGbl_RootNodeStruct;
+ PrevNode = NewNode;
+
/*
* Name entered successfully. If entry in PreDefinedNames[] specifies
* an initial value, create the initial value.
@@ -286,7 +322,7 @@ AcpiNsRootInitialize (
NewNode->Value = ObjDesc->Method.ParamCount;
#else
- /* Mark this as a very SPECIAL method */
+ /* Mark this as a very SPECIAL method (_OSI) */
ObjDesc->Method.InfoFlags = ACPI_METHOD_INTERNAL_ONLY;
ObjDesc->Method.Dispatch.Implementation = AcpiUtOsiImplementation;
@@ -359,7 +395,6 @@ AcpiNsRootInitialize (
}
}
-
UnlockAndExit:
(void) AcpiUtReleaseMutex (ACPI_MTX_NAMESPACE);
diff --git a/source/components/namespace/nseval.c b/source/components/namespace/nseval.c
index 2ecba6396d41..af6a6d4530fb 100644
--- a/source/components/namespace/nseval.c
+++ b/source/components/namespace/nseval.c
@@ -160,13 +160,6 @@
#define _COMPONENT ACPI_NAMESPACE
ACPI_MODULE_NAME ("nseval")
-/* Local prototypes */
-
-static void
-AcpiNsExecModuleCode (
- ACPI_OPERAND_OBJECT *MethodObj,
- ACPI_EVALUATE_INFO *Info);
-
/*******************************************************************************
*
@@ -465,206 +458,3 @@ Cleanup:
Info->FullPathname = NULL;
return_ACPI_STATUS (Status);
}
-
-
-/*******************************************************************************
- *
- * FUNCTION: AcpiNsExecModuleCodeList
- *
- * PARAMETERS: None
- *
- * RETURN: None. Exceptions during method execution are ignored, since
- * we cannot abort a table load.
- *
- * DESCRIPTION: Execute all elements of the global module-level code list.
- * Each element is executed as a single control method.
- *
- * NOTE: With this option enabled, each block of detected executable AML
- * code that is outside of any control method is wrapped with a temporary
- * control method object and placed on a global list. The methods on this
- * list are executed below.
- *
- * This function executes the module-level code for all tables only after
- * all of the tables have been loaded. It is a legacy option and is
- * not compatible with other ACPI implementations. See AcpiNsLoadTable.
- *
- * This function will be removed when the legacy option is removed.
- *
- ******************************************************************************/
-
-void
-AcpiNsExecModuleCodeList (
- void)
-{
- ACPI_OPERAND_OBJECT *Prev;
- ACPI_OPERAND_OBJECT *Next;
- ACPI_EVALUATE_INFO *Info;
- UINT32 MethodCount = 0;
-
-
- ACPI_FUNCTION_TRACE (NsExecModuleCodeList);
-
-
- /* Exit now if the list is empty */
-
- Next = AcpiGbl_ModuleCodeList;
- if (!Next)
- {
- ACPI_DEBUG_PRINT ((ACPI_DB_INIT_NAMES,
- "Legacy MLC block list is empty\n"));
-
- return_VOID;
- }
-
- /* Allocate the evaluation information block */
-
- Info = ACPI_ALLOCATE (sizeof (ACPI_EVALUATE_INFO));
- if (!Info)
- {
- return_VOID;
- }
-
- /* Walk the list, executing each "method" */
-
- while (Next)
- {
- Prev = Next;
- Next = Next->Method.Mutex;
-
- /* Clear the link field and execute the method */
-
- Prev->Method.Mutex = NULL;
- AcpiNsExecModuleCode (Prev, Info);
- MethodCount++;
-
- /* Delete the (temporary) method object */
-
- AcpiUtRemoveReference (Prev);
- }
-
- ACPI_INFO ((
- "Executed %u blocks of module-level executable AML code",
- MethodCount));
-
- ACPI_FREE (Info);
- AcpiGbl_ModuleCodeList = NULL;
- return_VOID;
-}
-
-
-/*******************************************************************************
- *
- * FUNCTION: AcpiNsExecModuleCode
- *
- * PARAMETERS: MethodObj - Object container for the module-level code
- * Info - Info block for method evaluation
- *
- * RETURN: None. Exceptions during method execution are ignored, since
- * we cannot abort a table load.
- *
- * DESCRIPTION: Execute a control method containing a block of module-level
- * executable AML code. The control method is temporarily
- * installed to the root node, then evaluated.
- *
- ******************************************************************************/
-
-static void
-AcpiNsExecModuleCode (
- ACPI_OPERAND_OBJECT *MethodObj,
- ACPI_EVALUATE_INFO *Info)
-{
- ACPI_OPERAND_OBJECT *ParentObj;
- ACPI_NAMESPACE_NODE *ParentNode;
- ACPI_OBJECT_TYPE Type;
- ACPI_STATUS Status;
-
-
- ACPI_FUNCTION_TRACE (NsExecModuleCode);
-
-
- /*
- * Get the parent node. We cheat by using the NextObject field
- * of the method object descriptor.
- */
- ParentNode = ACPI_CAST_PTR (
- ACPI_NAMESPACE_NODE, MethodObj->Method.NextObject);
- Type = AcpiNsGetType (ParentNode);
-
- /*
- * Get the region handler and save it in the method object. We may need
- * this if an operation region declaration causes a _REG method to be run.
- *
- * We can't do this in AcpiPsLinkModuleCode because
- * AcpiGbl_RootNode->Object is NULL at PASS1.
- */
- if ((Type == ACPI_TYPE_DEVICE) && ParentNode->Object)
- {
- MethodObj->Method.Dispatch.Handler =
- ParentNode->Object->Device.Handler;
- }
-
- /* Must clear NextObject (AcpiNsAttachObject needs the field) */
-
- MethodObj->Method.NextObject = NULL;
-
- /* Initialize the evaluation information block */
-
- memset (Info, 0, sizeof (ACPI_EVALUATE_INFO));
- Info->PrefixNode = ParentNode;
-
- /*
- * Get the currently attached parent object. Add a reference,
- * because the ref count will be decreased when the method object
- * is installed to the parent node.
- */
- ParentObj = AcpiNsGetAttachedObject (ParentNode);
- if (ParentObj)
- {
- AcpiUtAddReference (ParentObj);
- }
-
- /* Install the method (module-level code) in the parent node */
-
- Status = AcpiNsAttachObject (ParentNode, MethodObj, ACPI_TYPE_METHOD);
- if (ACPI_FAILURE (Status))
- {
- goto Exit;
- }
-
- /* Execute the parent node as a control method */
-
- Status = AcpiNsEvaluate (Info);
-
- ACPI_DEBUG_PRINT ((ACPI_DB_INIT_NAMES,
- "Executed module-level code at %p\n",
- MethodObj->Method.AmlStart));
-
- /* Delete a possible implicit return value (in slack mode) */
-
- if (Info->ReturnObject)
- {
- AcpiUtRemoveReference (Info->ReturnObject);
- }
-
- /* Detach the temporary method object */
-
- AcpiNsDetachObject (ParentNode);
-
- /* Restore the original parent object */
-
- if (ParentObj)
- {
- Status = AcpiNsAttachObject (ParentNode, ParentObj, Type);
- }
- else
- {
- ParentNode->Type = (UINT8) Type;
- }
-
-Exit:
- if (ParentObj)
- {
- AcpiUtRemoveReference (ParentObj);
- }
- return_VOID;
-}
diff --git a/source/components/namespace/nsinit.c b/source/components/namespace/nsinit.c
index 4dd28fd4e69e..51f98339489f 100644
--- a/source/components/namespace/nsinit.c
+++ b/source/components/namespace/nsinit.c
@@ -212,29 +212,30 @@ AcpiNsInitializeObjects (
ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH,
"**** Starting initialization of namespace objects ****\n"));
ACPI_DEBUG_PRINT_RAW ((ACPI_DB_INIT,
- "Completing Region/Field/Buffer/Package initialization:\n"));
+ "Final data object initialization: "));
- /* Set all init info to zero */
+ /* Clear the info block */
memset (&Info, 0, sizeof (ACPI_INIT_WALK_INFO));
/* Walk entire namespace from the supplied root */
+ /*
+ * TBD: will become ACPI_TYPE_PACKAGE as this type object
+ * is now the only one that supports deferred initialization
+ * (forward references).
+ */
Status = AcpiWalkNamespace (ACPI_TYPE_ANY, ACPI_ROOT_OBJECT,
- ACPI_UINT32_MAX, AcpiNsInitOneObject, NULL,
- &Info, NULL);
+ ACPI_UINT32_MAX, AcpiNsInitOneObject, NULL, &Info, NULL);
if (ACPI_FAILURE (Status))
{
ACPI_EXCEPTION ((AE_INFO, Status, "During WalkNamespace"));
}
ACPI_DEBUG_PRINT_RAW ((ACPI_DB_INIT,
- " Initialized %u/%u Regions %u/%u Fields %u/%u "
- "Buffers %u/%u Packages (%u nodes)\n",
- Info.OpRegionInit, Info.OpRegionCount,
- Info.FieldInit, Info.FieldCount,
- Info.BufferInit, Info.BufferCount,
- Info.PackageInit, Info.PackageCount, Info.ObjectCount));
+ "Namespace contains %u (0x%X) objects\n",
+ Info.ObjectCount,
+ Info.ObjectCount));
ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH,
"%u Control Methods found\n%u Op Regions found\n",
@@ -561,35 +562,19 @@ AcpiNsInitOneObject (
AcpiExEnterInterpreter ();
/*
- * Each of these types can contain executable AML code within the
- * declaration.
+ * Only initialization of Package objects can be deferred, in order
+ * to support forward references.
*/
switch (Type)
{
- case ACPI_TYPE_REGION:
-
- Info->OpRegionInit++;
- Status = AcpiDsGetRegionArguments (ObjDesc);
- break;
-
- case ACPI_TYPE_BUFFER_FIELD:
-
- Info->FieldInit++;
- Status = AcpiDsGetBufferFieldArguments (ObjDesc);
- break;
-
case ACPI_TYPE_LOCAL_BANK_FIELD:
+ /* TBD: BankFields do not require deferred init, remove this code */
+
Info->FieldInit++;
Status = AcpiDsGetBankFieldArguments (ObjDesc);
break;
- case ACPI_TYPE_BUFFER:
-
- Info->BufferInit++;
- Status = AcpiDsGetBufferArguments (ObjDesc);
- break;
-
case ACPI_TYPE_PACKAGE:
/* Complete the initialization/resolution of the package object */
@@ -600,8 +585,12 @@ AcpiNsInitOneObject (
default:
- /* No other types can get here */
+ /* No other types should get here */
+ Status = AE_TYPE;
+ ACPI_EXCEPTION ((AE_INFO, Status,
+ "Opcode is not deferred [%4.4s] (%s)",
+ AcpiUtGetNodeName (Node), AcpiUtGetTypeName (Type)));
break;
}
diff --git a/source/components/namespace/nsload.c b/source/components/namespace/nsload.c
index f5a82347af92..f7aaa11d2277 100644
--- a/source/components/namespace/nsload.c
+++ b/source/components/namespace/nsload.c
@@ -268,18 +268,6 @@ Unlock:
ACPI_DEBUG_PRINT ((ACPI_DB_INFO,
"**** Completed Table Object Initialization\n"));
- /*
- * This case handles the legacy option that groups all module-level
- * code blocks together and defers execution until all of the tables
- * are loaded. Execute all of these blocks at this time.
- * Execute any module-level code that was detected during the table
- * load phase.
- *
- * Note: this option is deprecated and will be eliminated in the
- * future. Use of this option can cause problems with AML code that
- * depends upon in-order immediate execution of module-level code.
- */
- AcpiNsExecModuleCodeList ();
return_ACPI_STATUS (Status);
}
diff --git a/source/components/namespace/nsutils.c b/source/components/namespace/nsutils.c
index 85732ff8b7bd..581ad5f3fd43 100644
--- a/source/components/namespace/nsutils.c
+++ b/source/components/namespace/nsutils.c
@@ -802,24 +802,11 @@ AcpiNsTerminate (
void)
{
ACPI_STATUS Status;
- ACPI_OPERAND_OBJECT *Prev;
- ACPI_OPERAND_OBJECT *Next;
ACPI_FUNCTION_TRACE (NsTerminate);
- /* Delete any module-level code blocks */
-
- Next = AcpiGbl_ModuleCodeList;
- while (Next)
- {
- Prev = Next;
- Next = Next->Method.Mutex;
- Prev->Method.Mutex = NULL; /* Clear the Mutex (cheated) field */
- AcpiUtRemoveReference (Prev);
- }
-
/*
* Free the entire namespace -- all nodes and all objects
* attached to the nodes
diff --git a/source/components/tables/tbdata.c b/source/components/tables/tbdata.c
index 9d86fdbc70ba..ba0c9108d53e 100644
--- a/source/components/tables/tbdata.c
+++ b/source/components/tables/tbdata.c
@@ -1191,19 +1191,6 @@ AcpiTbLoadTable (
Status = AcpiNsLoadTable (TableIndex, ParentNode);
/*
- * This case handles the legacy option that groups all module-level
- * code blocks together and defers execution until all of the tables
- * are loaded. Execute all of these blocks at this time.
- * Execute any module-level code that was detected during the table
- * load phase.
- *
- * Note: this option is deprecated and will be eliminated in the
- * future. Use of this option can cause problems with AML code that
- * depends upon in-order immediate execution of module-level code.
- */
- AcpiNsExecModuleCodeList ();
-
- /*
* Update GPEs for any new _Lxx/_Exx methods. Ignore errors. The host is
* responsible for discovering any new wake GPEs by running _PRW methods
* that may have been loaded by this table.
diff --git a/source/components/tables/tbxfload.c b/source/components/tables/tbxfload.c
index 217d54bf0a60..1e17db6c8c93 100644
--- a/source/components/tables/tbxfload.c
+++ b/source/components/tables/tbxfload.c
@@ -479,6 +479,13 @@ AcpiLoadTable (
ACPI_INFO (("Host-directed Dynamic ACPI Table Load:"));
Status = AcpiTbInstallAndLoadTable (ACPI_PTR_TO_PHYSADDR (Table),
ACPI_TABLE_ORIGIN_EXTERNAL_VIRTUAL, FALSE, &TableIndex);
+ if (ACPI_SUCCESS (Status))
+ {
+ /* Complete the initialization/resolution of new objects */
+
+ AcpiNsInitializeObjects ();
+ }
+
return_ACPI_STATUS (Status);
}
diff --git a/source/components/utilities/utinit.c b/source/components/utilities/utinit.c
index d83d7dabad1e..3e7ed41fe35c 100644
--- a/source/components/utilities/utinit.c
+++ b/source/components/utilities/utinit.c
@@ -342,7 +342,6 @@ AcpiUtInitGlobals (
/* Namespace */
- AcpiGbl_ModuleCodeList = NULL;
AcpiGbl_RootNode = NULL;
AcpiGbl_RootNodeStruct.Name.Integer = ACPI_ROOT_NAME;
AcpiGbl_RootNodeStruct.DescriptorType = ACPI_DESC_TYPE_NAMED;
diff --git a/source/components/utilities/utxfinit.c b/source/components/utilities/utxfinit.c
index f0c46c027bd7..d0b49bba86f1 100644
--- a/source/components/utilities/utxfinit.c
+++ b/source/components/utilities/utxfinit.c
@@ -381,24 +381,17 @@ AcpiInitializeObjects (
ACPI_FUNCTION_TRACE (AcpiInitializeObjects);
+#ifdef ACPI_OBSOLETE_BEHAVIOR
/*
- * This case handles the legacy option that groups all module-level
- * code blocks together and defers execution until all of the tables
- * are loaded. Execute all of these blocks at this time.
- * Execute any module-level code that was detected during the table
- * load phase.
- *
- * Note: this option is deprecated and will be eliminated in the
- * future. Use of this option can cause problems with AML code that
- * depends upon in-order immediate execution of module-level code.
+ * 05/2019: Removed, initialization now happens at both object
+ * creation and table load time
*/
- AcpiNsExecModuleCodeList ();
/*
* Initialize the objects that remain uninitialized. This
* runs the executable AML that may be part of the
- * declaration of these objects:
- * OperationRegions, BufferFields, Buffers, and Packages.
+ * declaration of these objects: OperationRegions, BufferFields,
+ * BankFields, Buffers, and Packages.
*/
if (!(Flags & ACPI_NO_OBJECT_INIT))
{
@@ -408,6 +401,7 @@ AcpiInitializeObjects (
return_ACPI_STATUS (Status);
}
}
+#endif
/*
* Initialize all device/region objects in the namespace. This runs
diff --git a/source/include/acevents.h b/source/include/acevents.h
index f10923421b29..c2eb7324b25c 100644
--- a/source/include/acevents.h
+++ b/source/include/acevents.h
@@ -245,7 +245,8 @@ AcpiEvMaskGpe (
ACPI_STATUS
AcpiEvAddGpeReference (
- ACPI_GPE_EVENT_INFO *GpeEventInfo);
+ ACPI_GPE_EVENT_INFO *GpeEventInfo,
+ BOOLEAN ClearOnEnable);
ACPI_STATUS
AcpiEvRemoveGpeReference (
diff --git a/source/include/acglobal.h b/source/include/acglobal.h
index 78bc7ca0be1c..e9e3206af0d7 100644
--- a/source/include/acglobal.h
+++ b/source/include/acglobal.h
@@ -323,7 +323,6 @@ ACPI_GLOBAL (BOOLEAN, AcpiGbl_VerboseLeakDump);
ACPI_GLOBAL (ACPI_NAMESPACE_NODE, AcpiGbl_RootNodeStruct);
ACPI_GLOBAL (ACPI_NAMESPACE_NODE *, AcpiGbl_RootNode);
ACPI_GLOBAL (ACPI_NAMESPACE_NODE *, AcpiGbl_FadtGpeDevice);
-ACPI_GLOBAL (ACPI_OPERAND_OBJECT *, AcpiGbl_ModuleCodeList);
extern const UINT8 AcpiGbl_NsProperties [ACPI_NUM_NS_TYPES];
extern const ACPI_PREDEFINED_NAMES AcpiGbl_PreDefinedNames [NUM_PREDEFINED_NAMES];
diff --git a/source/include/acnamesp.h b/source/include/acnamesp.h
index eaa2f1b4e898..6d148fea83ba 100644
--- a/source/include/acnamesp.h
+++ b/source/include/acnamesp.h
@@ -419,10 +419,6 @@ ACPI_STATUS
AcpiNsEvaluate (
ACPI_EVALUATE_INFO *Info);
-void
-AcpiNsExecModuleCodeList (
- void);
-
/*
* nsarguments - Argument count/type checking for predefined/reserved names
diff --git a/source/include/acpixf.h b/source/include/acpixf.h
index 8778da7241b0..2893bf23b1ca 100644
--- a/source/include/acpixf.h
+++ b/source/include/acpixf.h
@@ -154,7 +154,7 @@
/* Current ACPICA subsystem version in YYYYMMDD format */
-#define ACPI_CA_VERSION 0x20190509
+#define ACPI_CA_VERSION 0x20190703
#include "acconfig.h"
#include "actypes.h"
diff --git a/source/include/platform/acmsvc.h b/source/include/platform/acmsvc.h
index d39fed71a708..394b8095cf58 100644
--- a/source/include/platform/acmsvc.h
+++ b/source/include/platform/acmsvc.h
@@ -256,6 +256,10 @@
#endif
#endif
+/* warn C4001: use of slash-slash comments */
+/* NOTE: MSVC 2015 headers use these extensively */
+#pragma warning(disable:4001)
+
/* warn C4100: unreferenced formal parameter */
#pragma warning(disable:4100)
@@ -268,10 +272,21 @@
/* warn C4131: uses old-style declarator (iASL compiler only) */
#pragma warning(disable:4131)
+/* warn C4131: uses old-style declarator (iASL compiler only) */
+#pragma warning(disable:4459)
+
#if _MSC_VER > 1200 /* Versions above VC++ 6 */
#pragma warning( disable : 4295 ) /* needed for acpredef.h array */
#endif
+/*
+ * MSVC 2015+
+ */
+
+ /* warn C4459: xxxx (identifier) hides global declaration */
+#pragma warning(disable:4459)
+
+
/* Debug support. */
diff --git a/source/include/platform/acwin.h b/source/include/platform/acwin.h
index 5071f948645a..51b28ca808ae 100644
--- a/source/include/platform/acwin.h
+++ b/source/include/platform/acwin.h
@@ -152,9 +152,33 @@
#ifndef __ACWIN_H__
#define __ACWIN_H__
+#include <io.h>
+
#define ACPI_USE_STANDARD_HEADERS
#define ACPI_USE_SYSTEM_CLIBRARY
+ /* Note: do not include any C library headers here */
+
+ /*
+ * Note: MSVC project files should define ACPI_DEBUGGER and ACPI_DISASSEMBLER
+ * as appropriate to enable editor functions like "Find all references".
+ * The editor isn't smart enough to dig through the include files to find
+ * out if these are actually defined.
+ */
+
+ /* Eliminate warnings for "old" (non-secure) versions of clib functions */
+
+#ifndef _CRT_SECURE_NO_WARNINGS
+#define _CRT_SECURE_NO_WARNINGS
+#endif
+
+/* Eliminate warnings for POSIX clib function names (open, write, etc.) */
+
+#ifndef _CRT_NONSTDC_NO_DEPRECATE
+#define _CRT_NONSTDC_NO_DEPRECATE
+#endif
+
+
#define ACPI_MACHINE_WIDTH 32
#define ACPI_USE_NATIVE_DIVIDE
#define ACPI_USE_NATIVE_MATH64
@@ -183,7 +207,9 @@ typedef COMPILER_DEPENDENT_UINT64 u64;
#define stat _stat
#define fstat _fstat
#define mkdir _mkdir
-#define snprintf _snprintf
+#define fileno _fileno
+#define isatty _isatty
+
#if _MSC_VER <= 1200 /* Versions below VC++ 6 */
#define vsnprintf _vsnprintf
#endif
@@ -195,6 +221,9 @@ typedef COMPILER_DEPENDENT_UINT64 u64;
#define S_IREAD _S_IREAD
#define S_IWRITE _S_IWRITE
#define S_IFDIR _S_IFDIR
+#if _MSC_VER < 1900
+#define snprintf _snprintf
+#endif
/*
diff --git a/source/include/platform/acwin64.h b/source/include/platform/acwin64.h
index beb331d0e7fd..9519c0395328 100644
--- a/source/include/platform/acwin64.h
+++ b/source/include/platform/acwin64.h
@@ -155,6 +155,28 @@
#define ACPI_USE_STANDARD_HEADERS
#define ACPI_USE_SYSTEM_CLIBRARY
+ /* Note: do not include any C library headers here */
+
+ /*
+ * Note: MSVC project files should define ACPI_DEBUGGER and ACPI_DISASSEMBLER
+ * as appropriate to enable editor functions like "Find all references".
+ * The editor isn't smart enough to dig through the include files to find
+ * out if these are actually defined.
+ */
+
+ /* Eliminate warnings for "old" (non-secure) versions of clib functions */
+
+#ifndef _CRT_SECURE_NO_WARNINGS
+#define _CRT_SECURE_NO_WARNINGS
+#endif
+
+/* Eliminate warnings for POSIX clib function names (open, write, etc.) */
+
+#ifndef _CRT_NONSTDC_NO_DEPRECATE
+#define _CRT_NONSTDC_NO_DEPRECATE
+#endif
+
+
#define ACPI_MACHINE_WIDTH 64
/*
diff --git a/source/tools/acpiexec/aetests.c b/source/tools/acpiexec/aetests.c
index 9f35e8f5acaa..f11a61283655 100644
--- a/source/tools/acpiexec/aetests.c
+++ b/source/tools/acpiexec/aetests.c
@@ -629,7 +629,7 @@ AeLateTest (
*****************************************************************************/
static void
-AeGlobalAddressRangeCheck(
+AeGlobalAddressRangeCheck (
void)
{
ACPI_STATUS Status;
@@ -638,18 +638,18 @@ AeGlobalAddressRangeCheck(
UINT32 i;
- ReturnBuffer.Length = ACPI_ALLOCATE_BUFFER;
- AcpiUtInitializeBuffer (&ReturnBuffer, ACPI_ALLOCATE_BUFFER);
-
for (i = 0; i < ACPI_ADDRESS_RANGE_MAX; i++)
{
Current = AcpiGbl_AddressRangeList[i];
while (Current)
{
+ ReturnBuffer.Length = ACPI_ALLOCATE_BUFFER;
+
Status = AcpiGetName (Current->RegionNode, ACPI_SINGLE_NAME, &ReturnBuffer);
ACPI_CHECK_OK (AcpiGetname, Status);
+ AcpiOsFree (ReturnBuffer.Pointer);
Current = Current->Next;
}
}
diff --git a/source/tools/acpinames/anstubs.c b/source/tools/acpinames/anstubs.c
index d96df4a77e48..af6cb9f8ea6e 100644
--- a/source/tools/acpinames/anstubs.c
+++ b/source/tools/acpinames/anstubs.c
@@ -302,13 +302,6 @@ AcpiNsEvaluate (
}
void
-AcpiNsExecModuleCodeList (
- void)
-{
- return;
-}
-
-void
AcpiExDoDebugObject (
ACPI_OPERAND_OBJECT *SourceDesc,
UINT32 Level,
diff --git a/source/tools/acpixtract/axmain.c b/source/tools/acpixtract/axmain.c
index c53b39b3b381..a027f786c866 100644
--- a/source/tools/acpixtract/axmain.c
+++ b/source/tools/acpixtract/axmain.c
@@ -196,7 +196,7 @@ DisplayUsage (
*
******************************************************************************/
-int
+int ACPI_SYSTEM_XFACE
main (
int argc,
char *argv[])
diff --git a/source/compiler/aslload.c b/source/compiler/aslload.c
index d1523ab69cc8..4dd2dfd0f561 100644
--- a/source/compiler/aslload.c
+++ b/source/compiler/aslload.c
@@ -164,6 +164,7 @@
static ACPI_STATUS
LdLoadFieldElements (
+ UINT32 AmlType,
ACPI_PARSE_OBJECT *Op,
ACPI_WALK_STATE *WalkState);
@@ -190,6 +191,10 @@ LdCommonNamespaceEnd (
UINT32 Level,
void *Context);
+static void
+LdCheckSpecialNames (
+ ACPI_NAMESPACE_NODE *Node,
+ ACPI_PARSE_OBJECT *Op);
/*******************************************************************************
*
@@ -247,7 +252,8 @@ LdLoadNamespace (
*
* FUNCTION: LdLoadFieldElements
*
* PARAMETERS: Op - Parent node (Field)
* WalkState - Current walk state
*
* RETURN: Status
@@ -259,6 +265,7 @@ LdLoadNamespace (
static ACPI_STATUS
LdLoadFieldElements (
+ UINT32 AmlType,
ACPI_PARSE_OBJECT *Op,
ACPI_WALK_STATE *WalkState)
{
@@ -274,7 +281,7 @@ LdLoadFieldElements (
{
Status = AcpiNsLookup (WalkState->ScopeInfo,
SourceRegion->Asl.Value.String,
- ACPI_TYPE_REGION, ACPI_IMODE_EXECUTE,
+ AmlType, ACPI_IMODE_EXECUTE,
ACPI_NS_DONT_OPEN_SCOPE, NULL, &Node);
if (Status == AE_NOT_FOUND)
{
@@ -507,11 +514,15 @@ LdNamespace1Begin (
*/
switch (Op->Asl.AmlOpcode)
{
- case AML_BANK_FIELD_OP:
case AML_INDEX_FIELD_OP:
+
+ Status = LdLoadFieldElements (ACPI_TYPE_LOCAL_REGION_FIELD, Op, WalkState);
+ return (Status);
+
+ case AML_BANK_FIELD_OP:
case AML_FIELD_OP:
- Status = LdLoadFieldElements (Op, WalkState);
+ Status = LdLoadFieldElements (ACPI_TYPE_REGION, Op, WalkState);
return (Status);
case AML_INT_CONNECTION_OP:
@@ -966,6 +977,10 @@ LdNamespace1Begin (
}
}
+ /* Check special names like _WAK and _PTS */
+
+ LdCheckSpecialNames (Node, Op);
+
if (ForceNewScope)
{
Status = AcpiDsScopeStackPush (Node, ObjectType, WalkState);
@@ -1006,6 +1021,42 @@ FinishNode:
/*******************************************************************************
*
+ * FUNCTION: LdCheckSpecialNames
+ *
+ * PARAMETERS: Node - Node that represents the named object
+ * Op - Named object declaring this named object
+ *
+ * RETURN: None
+ *
+ * DESCRIPTION: Check if certain named objects are declared in the incorrect
+ * scope. Special named objects are listed in
+ * AslGbl_SpecialNamedObjects and can only be declared at the root
+ * scope.
+ *
+ ******************************************************************************/
+
+static void
+LdCheckSpecialNames (
+ ACPI_NAMESPACE_NODE *Node,
+ ACPI_PARSE_OBJECT *Op)
+{
+ UINT32 i;
+
+
+ for (i = 0; i < MAX_SPECIAL_NAMES; i++)
+ {
+ if (ACPI_COMPARE_NAMESEG(Node->Name.Ascii, AslGbl_SpecialNamedObjects[i]) &&
+ Node->Parent != AcpiGbl_RootNode)
+ {
+ AslError (ASL_ERROR, ASL_MSG_INVALID_SPECIAL_NAME, Op, Op->Asl.ExternalName);
+ return;
+ }
+ }
+}
+
+
+/*******************************************************************************
+ *
* FUNCTION: LdNamespace2Begin
*
* PARAMETERS: ASL_WALK_CALLBACK
@@ -247,7 +252,8 @@ LdLoadNamespace (
* FUNCTION: LdLoadFieldElements
*
- * PARAMETERS: Op - Parent node (Field)
+ * PARAMETERS: AmlType - Type to search
+ * Op - Parent node (Field)
* WalkState - Current walk state
*